module Treestructs2 where

--I'm truncating my tree height
maxHeight :: Int
maxHeight = 6

--sets the viewing distances for each of the layers
viewRange :: [Int]
viewRange = [3,4,4,4,4,4,4,4]

-- defining my tree data type: 
-- "a tree is either a fork, a leaf, or a player" (all labelled)
data Tree a = Fork a (Tree a) (Tree a) | Leaf a | Player a 
             deriving Show

--appends one tree to the 'player' node of another
append :: Tree a -> Tree a -> Tree a
append s (Fork n t u) = Fork n (append s t) (append s u)
append s (Player a) = s
append s (Leaf a) = Leaf a

--the main function.  how the binary tree is generated 
--depends on whether the player is actually in it or not...
makeTree :: Int -> Tree Int
makeTree n = 
    if (0<=n) 
    then if (n<2^(maxHeight)) 
         then intToTree n 0 n
         else goingDown (0) (maxHeight) n
    else goingDown (-n) (maxHeight) n


--generates the tree structure upwards, starting from the player
intToTree :: Int -> Int -> Int -> Tree Int
intToTree n 6 playerpos = Player n
intToTree n h playerpos = if ((n `mod` 2)==0)
              then
                  append (Fork n (Player (n*(2^h))) (goingDown ((n+1)*(2^h)) h playerpos)) (intToTree (n `div` 2) (h+1) playerpos )
              else
                  append (Fork (n-1) (goingDown ((n-1)*(2^h)) h playerpos) (Player (n*(2^h)))) (intToTree (n `div` 2) (h+1) playerpos )

--awfully suboptimal, just sets first digit to 0
choplsd :: Int -> Int
choplsd n = (n `div` 2)*2

--fills out the visible branches, goes downwards
goingDown :: Int -> Int -> Int -> Tree Int
goingDown n 0 playerpos = Leaf n
goingDown n h playerpos = if abs(choplsd n - choplsd playerpos) <= (viewRange!!h)*2^h
                              then 
                                  Fork n (goingDown n (h-1) playerpos)  (goingDown (n+2^(h-1)) (h-1) playerpos) 
                              else 
                                  Leaf n